package  org.java.evolutionary.sequence.func;

import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;

import ec.EvolutionState;
import ec.Problem;
import ec.app.sequence.MotifData;
import  org.java.evolutionary.sequence.SequenceClassificationProblem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
/**
 * This can be used for generating Explicit Correlational Features. Implcit are generated by not using
 * these but having AND which can take two positional etc. 
 * This takes 4 parameters, 2 Motifs for comparison, startPosition and closeness integer (1,2,etc) 
 * Example AGGT, TAC, 24,3 would mean check for correlation of having AGCT @ 24 position and TAC at 27
 * @author udaykamath
 *
 */
public class Correlational extends GPNode{
	
	public String toString() {
		return " correlate ";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 4)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}
	
	

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		MotifData motifData = ((MotifData) (input));
		//child should give me motif
		children[0].eval(state, thread, input, stack, individual, problem);		
		//get the curernt motif
		char[] currentMotif = motifData.currentMotif;
		children[1].eval(state, thread, input, stack, individual, problem);
		char[] anotherMotif = motifData.currentMotif;
		children[2].eval(state, thread, input, stack, individual, problem);
		int currentPosition = motifData.currentPosition;
		children[3].eval(state, thread, input, stack, individual, problem);
		int close = motifData.close;
		//check for match using biojava
		SequenceClassificationProblem spliceSetProblem =(SequenceClassificationProblem)problem;
		PatternFactory factory = spliceSetProblem.getFactory(thread);
		SymbolList sequence = spliceSetProblem.getCurrentSequence();
		boolean matches = matches(sequence,currentMotif,anotherMotif,currentPosition,close,factory);
		motifData.matches = matches;
	}
	
	private boolean matches(SymbolList seq, char[] motif1, char[] motif2, int position, int proximity,PatternFactory factory  ){
		boolean matched = false;
		try{
			//in the sequence find the match
			Pattern bp1 = factory.compile(new String(motif1));
			Pattern bp2 = factory.compile(new String(motif2));
			Matcher occurences = bp1.matcher(seq);
			while (occurences.find()) {
				int position1 = occurences.start();
				if (position1 == position) {
					occurences = bp2.matcher(seq);
					while (occurences.find()) {
						int position2 = occurences.start();
						if((position2 >= position1+ motif1.length) && Math.abs((position1 + motif1.length)-position2) <= proximity
								||
								((position1 >= position2+ motif2.length) &&Math.abs((position2 + motif2.length)-position1) <= proximity)){	
							matched = true;
							break;
						}
					}
					break;
				}
			}	    
		}catch(Exception e){
			e.printStackTrace();
		}
		return matched;
	}
}

