package org.java.featurebased;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import org.biojava.bio.seq.DNATools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.io.SeqIOTools;
import org.biojava.bio.symbol.FiniteAlphabet;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;
import org.java.util.FileUtil;

public abstract class AbstractFeatureBased implements FeatureBasedMotif{
	protected HashSet<String> features = new HashSet<String>();
	protected String positiveFile;
	protected boolean fastaPositive;
	protected String negativeFile;
	protected boolean fastaNegative;
	protected String outputFile;
	protected int motifLength;
	
	public AbstractFeatureBased(String positiveFile, boolean fastaPositive,String negativeFile, boolean fastaNegative, String outputFile, int motifLength){
		this.positiveFile=positiveFile;
		this.fastaPositive= fastaPositive;
		this.negativeFile=negativeFile;
		this.fastaNegative=fastaNegative;
		this.outputFile=outputFile;
		this.motifLength=motifLength;
	}
	
	public void changeForTest(String positiveFile, String negativeFile, String outputFile){
		this.positiveFile = positiveFile;
		this.negativeFile = negativeFile;
		this.outputFile = outputFile;
		
	}
	
	/**
	 * Given positive, negative files and kmers does quick file generation
	 * @param pos
	 * @param neg
	 * @param libSvmFile
	 * @throws Exception
	 */
	public void generateLibSVMFile() throws Exception{
		//generate features first
		this.generateFeatures();
		File libsvmFile = new File(outputFile);
		if(fastaPositive){
			BufferedReader buffer = new BufferedReader(new FileReader(
						positiveFile));			
				SequenceIterator fileIterator = (SequenceIterator) SeqIOTools
				.fileToBiojava("fasta", "DNA", buffer);
				int done =0;
				while (fileIterator.hasNext()) {
					Sequence seq = fileIterator.nextSequence();		
					String svmFeatures = this.createKernelVector(seq.seqString());
					//append the label
					svmFeatures = "1" + svmFeatures + "\n";
					FileUtil.writeStringToTextFile(libsvmFile, svmFeatures, true);
					System.out.println("Done " + done);
					done++;
				}		
		}
		else{
			BufferedReader buffer = new BufferedReader(
					new FileReader(positiveFile));
			String totalLine = buffer.readLine();
			
			int done = 0;
			while (totalLine != null)
            {
                String sequence, label;
                int startOfSpace = totalLine.indexOf(" ");
                String token1 = totalLine.substring(0, startOfSpace);
                String token2 = totalLine.substring(startOfSpace + 1, totalLine.length());
                if (token1.equals("1") || token1.equals("-1"))
                     { label = token1; sequence = token2; }
                else { label = token2; sequence = token1; }
//				String sequence = totalLine;
                String svmFeatures = this.createKernelVector(sequence);
                svmFeatures = "1" + svmFeatures + "\n";
				FileUtil.writeStringToTextFile(libsvmFile, svmFeatures, true);
				System.out.println("Done " + done);
				done++;
				totalLine = buffer.readLine();
			}
		}
		if(fastaNegative){
			BufferedReader buffer = new BufferedReader(new FileReader(
						negativeFile));			
				SequenceIterator fileIterator = (SequenceIterator) SeqIOTools
				.fileToBiojava("fasta", "DNA", buffer);
				int done =0;
				while (fileIterator.hasNext()) {
					Sequence seq = fileIterator.nextSequence();		
					String svmFeatures = this.createKernelVector(seq.seqString());
					//append the label
					svmFeatures = "-1" + svmFeatures + "\n";
					FileUtil.writeStringToTextFile(libsvmFile, svmFeatures, true);
					System.out.println("Done " + done);
					done++;
				}		
		}
		else{
			BufferedReader negativeBuffer = new BufferedReader(
					new FileReader(negativeFile));
			String totalLine = negativeBuffer.readLine();
			
			int done = 0;
			while (totalLine != null)
            {
                String sequence, label;
                int startOfSpace = totalLine.indexOf(" ");
                String token1 = totalLine.substring(0, startOfSpace);
                String token2 = totalLine.substring(startOfSpace + 1, totalLine.length());
                if (token1.equals("1") || token1.equals("-1"))
                     { label = token1; sequence = token2; }
                else { label = token2; sequence = token1; }
//				String sequence = totalLine;
                String svmFeatures = this.createKernelVector(sequence);
                svmFeatures = "-1" + svmFeatures + "\n";
				FileUtil.writeStringToTextFile(libsvmFile, svmFeatures, true);
				System.out.println("Done " + done);
				done++;
				totalLine = negativeBuffer.readLine();
			}
		}
		
	}
	
		
	public String createKernelVector(String sequence) throws Exception{
		FiniteAlphabet iupac = DNATools.getDNA();
		PatternFactory factory = PatternFactory.makeFactory(iupac);
		double[] featureCounts = new double[features.size()];
		int i =0;
		Iterator iterator = this.features.iterator();
		while(iterator.hasNext()){
			Pattern pattern = factory.compile((String)iterator.next());
			boolean match = this.matchPattern(pattern,sequence);			
			if(match)
				featureCounts[i] = 1;
			else
				featureCounts[i] = 0;
			i=i+1;
		}
		//create a vector
		StringBuffer vector = new StringBuffer();
		for(int p=0; p< featureCounts.length; p++){
			int feature = p+1;
			if(featureCounts[p] >0 )
				vector.append(" " + feature+ ":" + featureCounts[p]);
		}		
		return vector.toString();
	}
	
	

	private boolean matchPattern(Pattern pattern, String sequence) {
		boolean match = false;
		Matcher occurences;
		try {

			SymbolList workingSequence = DNATools.createDNA(sequence);
			// Obtain iterator of matches.
			occurences = pattern.matcher(workingSequence);
			// For each match increment the count
			 while(occurences.find()){		    	
		    		match = true;
		    		break;
		    	}	
		} catch (Exception e) {
			e.printStackTrace();
		}
		return match;
	}

}
