package org.java.statistics;

import de.jstacs.algorithms.optimization.Optimizer;
import de.jstacs.algorithms.optimization.termination.SmallDifferenceOfFunctionEvaluationsCondition;
import de.jstacs.classifiers.assessment.KFoldCrossValidation;
import de.jstacs.classifiers.assessment.KFoldCrossValidationAssessParameterSet;
import de.jstacs.classifiers.differentiableSequenceScoreBased.AbstractMultiThreadedOptimizableFunction;
import de.jstacs.classifiers.differentiableSequenceScoreBased.OptimizableFunction.KindOfParameter;
import de.jstacs.classifiers.differentiableSequenceScoreBased.gendismix.GenDisMixClassifier;
import de.jstacs.classifiers.differentiableSequenceScoreBased.gendismix.GenDisMixClassifierParameterSet;
import de.jstacs.classifiers.differentiableSequenceScoreBased.logPrior.CompositeLogPrior;
import de.jstacs.classifiers.differentiableSequenceScoreBased.msp.MSPClassifier;
import de.jstacs.classifiers.performanceMeasures.NumericalPerformanceMeasureParameterSet;
import de.jstacs.classifiers.performanceMeasures.PerformanceMeasureParameterSet;
import de.jstacs.classifiers.trainSMBased.TrainSMBasedClassifier;
import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DNADataSet;
import de.jstacs.data.DataSet;
import de.jstacs.data.DataSet.PartitionMethod;
import de.jstacs.data.alphabets.DNAAlphabetContainer;
import de.jstacs.io.SparseStringExtractor;
import de.jstacs.io.StringExtractor;
import de.jstacs.results.ListResult;
import de.jstacs.sequenceScores.statisticalModels.differentiable.DifferentiableStatisticalModel;
import de.jstacs.sequenceScores.statisticalModels.differentiable.DifferentiableStatisticalModelFactory;
import de.jstacs.sequenceScores.statisticalModels.differentiable.directedGraphicalModels.BayesianNetworkDiffSM;
import de.jstacs.sequenceScores.statisticalModels.differentiable.directedGraphicalModels.structureLearning.measures.InhomogeneousMarkov;
import de.jstacs.sequenceScores.statisticalModels.differentiable.mixture.MixtureDiffSM;
import de.jstacs.sequenceScores.statisticalModels.trainable.TrainableStatisticalModel;
import de.jstacs.sequenceScores.statisticalModels.trainable.TrainableStatisticalModelFactory;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.inhomogeneous.BayesianNetworkTrainSM;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.inhomogeneous.StructureLearner.LearningType;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.inhomogeneous.StructureLearner.ModelType;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.inhomogeneous.parameters.BayesianNetworkTrainSMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.mixture.MixtureTrainSM;
import de.jstacs.sequenceScores.statisticalModels.trainable.mixture.AbstractMixtureTrainSM.Parameterization;

public class StatisticalMethodsTest {
	
	

	public static void testBayesNetworkMixtureClassifier(String pos, String neg)
			throws Exception {
		// create datasets, PWWM needs smallest length
		DataSet[] data = new DataSet[2];
		data[0] = new DataSet(new DNADataSet(pos), 210);
		data[1] = new DataSet(new DNADataSet(neg), 210);

		int length = data[0].getElementLength();
		AlphabetContainer container = data[0].getAlphabetContainer();
		// equivalent sample size =^= ESS
		double essFg = 4, essBg = 4;
		// create DifferentiableSequenceScore, here PWM
		DifferentiableStatisticalModel pwmFg = new BayesianNetworkDiffSM(
				container, length, essFg, true, new InhomogeneousMarkov(0));
		DifferentiableStatisticalModel pwmBg = new BayesianNetworkDiffSM(
				container, length, essBg, true, new InhomogeneousMarkov(0));

		// create parameters of the classifier
		GenDisMixClassifierParameterSet cps = new GenDisMixClassifierParameterSet(
				container,// the used alphabets
				0,// sequence length that can be modeled/classified
				Optimizer.QUASI_NEWTON_BFGS, 1E-1, 1E-1, 1,// optimization
				// parameter
				false,// use free parameters or all
				KindOfParameter.PLUGIN,// how to start the numerical
				// optimization
				true,// use a normalized objective function
				// AbstractMultiThreadedOptimizableFunction
				// .getNumberOfAvailableProcessors()// number of compute
				// threads
				1);
		double[] beta = { 0.3, 0.3, 0.4 };
		GenDisMixClassifier classifier = new GenDisMixClassifier(cps,
				new CompositeLogPrior(), beta, pwmFg, pwmBg);
		// suited for two-class problems (e.g., AUC-ROC, AUC-PR)
		NumericalPerformanceMeasureParameterSet set = (NumericalPerformanceMeasureParameterSet) PerformanceMeasureParameterSet
				.createFilledParameters(true, 0.99, 0.95, 0.95, 1);

		// you might also want to do a cross-validation on your data to compare
		// these two classifiers
		KFoldCrossValidation cv = new KFoldCrossValidation(classifier);
		KFoldCrossValidationAssessParameterSet assessPS = new KFoldCrossValidationAssessParameterSet(
				PartitionMethod.PARTITION_BY_NUMBER_OF_SYMBOLS, 0, true, 10);
		ListResult lr = cv.assess(set, assessPS, data[0], data[1]);

		System.out.println(lr);
	}

	public static void testMSPClassifier(String pos, String neg)
			throws Exception {
		// create datasets, PWWM needs smallest length
		DataSet[] data = new DataSet[2];
		data[0] = new DataSet(new DNADataSet(pos), 210);
		data[1] = new DataSet(new DNADataSet(neg), 210);
		// discriminative case:
		// e.g., PWM models (for PWMs or iMMs, the training sequences must be of
		// identical length)
		DifferentiableStatisticalModel fgModel2 = DifferentiableStatisticalModelFactory
				.createPWM(DNAAlphabetContainer.SINGLETON, data[1]
						.getElementLength(), 2);
		// or hMM(3)
		DifferentiableStatisticalModel bgModel2 = DifferentiableStatisticalModelFactory
				.createHomogeneousMarkovModel(DNAAlphabetContainer.SINGLETON,
						4.0, 2, data[0].getElementLength());
		// discriminative classifier with these models
		// first define parameters for numerical optimization
		int length = data[0].getElementLength(); // If all your sequences
		// have same length, this
		// just be set to this
		// length. Otherwise, 0
		// indicates that sequences
		// may have different
		// lengths
		GenDisMixClassifierParameterSet params = new GenDisMixClassifierParameterSet(
				DNAAlphabetContainer.SINGLETON, length,
				Optimizer.QUASI_NEWTON_BFGS, 1E-6, 1E-4, 1E-4, false,
				KindOfParameter.PLUGIN, true, 1);
		MSPClassifier clDis = new MSPClassifier(params,
				new CompositeLogPrior(), fgModel2, bgModel2);
		// suited for two-class problems (e.g., AUC-ROC, AUC-PR)
		NumericalPerformanceMeasureParameterSet set = (NumericalPerformanceMeasureParameterSet) PerformanceMeasureParameterSet
				.createFilledParameters(true, 0.99, 0.95, 0.95, 1);

		// you might also want to do a cross-validation on your data to compare
		// these two classifiers
		KFoldCrossValidation cv = new KFoldCrossValidation(clDis);
		KFoldCrossValidationAssessParameterSet assessPS = new KFoldCrossValidationAssessParameterSet(
				PartitionMethod.PARTITION_BY_NUMBER_OF_SYMBOLS, 0, true, 10);
		ListResult lr = cv.assess(set, assessPS, data[0], data[1]);

		System.out.println(lr);
	}

	public static void testHomogenousHMM(String pos, String neg)
			throws Exception {
		// create datasets, PWWM needs smallest length
		DataSet[] data = new DataSet[2];
		AlphabetContainer container = DNAAlphabetContainer.SINGLETON;
		// create a DataSet using the alphabet from above in FastA-format
		data[0] = new DataSet(container, new SparseStringExtractor(pos,
				StringExtractor.FASTA));

		// create a DataSet using the alphabet from above
		data[1] = new DataSet(container, new SparseStringExtractor(neg,
				StringExtractor.FASTA));

		// create models
		TrainableStatisticalModel fgModel = TrainableStatisticalModelFactory
				.createHomogeneousMarkovModel(DNAAlphabetContainer.SINGLETON,
						4.0, (byte) 4);
		// background model: hMM(3)
		TrainableStatisticalModel bgModel = TrainableStatisticalModelFactory
				.createHomogeneousMarkovModel(DNAAlphabetContainer.SINGLETON,
						4.0, (byte) 4);
		// generative classifier with these models
		TrainSMBasedClassifier clGen = new TrainSMBasedClassifier(fgModel,
				bgModel);
		// suited for two-class problems (e.g., AUC-ROC, AUC-PR)
		NumericalPerformanceMeasureParameterSet set = (NumericalPerformanceMeasureParameterSet) PerformanceMeasureParameterSet
				.createFilledParameters(true, 0.99, 0.95, 0.95, 1);
		// perform kfold cross validation
		KFoldCrossValidation cv = new KFoldCrossValidation(clGen);
		KFoldCrossValidationAssessParameterSet assessPS = new KFoldCrossValidationAssessParameterSet(
				PartitionMethod.PARTITION_BY_NUMBER_OF_SYMBOLS, 0, true, 10);
		ListResult lr = cv.assess(set, assessPS, data[0], data[1]);
		System.out.println(lr);
	}

	public static void testInHomogenousHMM(String pos, String neg)
			throws Exception {
		// create datasets, PWWM needs smallest length
		DataSet[] data = new DataSet[2];
		AlphabetContainer container = DNAAlphabetContainer.SINGLETON;
		data[0] = new DataSet(new DNADataSet(pos), 120);
		data[1] = new DataSet(new DNADataSet(neg), 120);

		// create models
		TrainableStatisticalModel fgModel = TrainableStatisticalModelFactory.createInhomogeneousMarkovModel( container, 120, 4, (byte)4 );
		// background model: hMM(3)
		TrainableStatisticalModel bgModel = TrainableStatisticalModelFactory.createInhomogeneousMarkovModel( container, 120, 4, (byte)4 );
		// generative classifier with these models
		TrainSMBasedClassifier clGen = new TrainSMBasedClassifier(fgModel,
				bgModel);
		// suited for two-class problems (e.g., AUC-ROC, AUC-PR)
		NumericalPerformanceMeasureParameterSet set = (NumericalPerformanceMeasureParameterSet) PerformanceMeasureParameterSet
				.createFilledParameters(true, 0.99, 0.95, 0.95, 1);
		// perform kfold cross validation
		KFoldCrossValidation cv = new KFoldCrossValidation(clGen);
		KFoldCrossValidationAssessParameterSet assessPS = new KFoldCrossValidationAssessParameterSet(
				PartitionMethod.PARTITION_BY_NUMBER_OF_SYMBOLS, 0, true, 10);
		ListResult lr = cv.assess(set, assessPS, data[0], data[1]);
		System.out.println(lr);
	}

	public static void testPositionalWeightMatrix(String pos, String neg)
			throws Exception {
		// create datasets, PWWM needs smallest length
		DataSet[] data = new DataSet[2];
		data[0] = new DataSet(new DNADataSet(pos), 120);
		data[1] = new DataSet(new DNADataSet(neg), 120);
		AlphabetContainer con = data[0].getAlphabetContainer();
		// create models
		TrainableStatisticalModel fgModel = TrainableStatisticalModelFactory
				.createPWM(con, data[0].getElementLength(), (byte) 4);
		TrainableStatisticalModel bgModel = TrainableStatisticalModelFactory
				.createPWM(con, data[1].getElementLength(), (byte) 4);
		TrainSMBasedClassifier clGen = new TrainSMBasedClassifier(fgModel,
				bgModel);
		// suited for two-class problems (e.g., AUC-ROC, AUC-PR)
		NumericalPerformanceMeasureParameterSet set = (NumericalPerformanceMeasureParameterSet) PerformanceMeasureParameterSet
				.createFilledParameters(true, 0.99, 0.95, 0.95, 1);
		// perform kfold cross validation
		KFoldCrossValidation cv = new KFoldCrossValidation(clGen);
		KFoldCrossValidationAssessParameterSet assessPS = new KFoldCrossValidationAssessParameterSet(
				PartitionMethod.PARTITION_BY_NUMBER_OF_SYMBOLS, 0, true, 10);
		ListResult lr = cv.assess(set, assessPS, data[0], data[1]);
		System.out.println(lr);
	}

	public static void main(String[] args) throws Exception {
		StatisticalMethodsTest.testInHomogenousHMM(args[0],
				args[1]);
	}

}
